function out_vec = estimate_model(carrier,time_period,time_end)

clc 
rng(681);

DATA = csvread('iphone_matlab_2019_11_13.csv',1,0);

options = optimset('Display','final','MaxFunEval', 100000,'MaxIter', 100000, 'TolX', 1e-15);
optionsnp = optimset('Display','final','MaxFunEval', 100000,'MaxIter', 100000, 'TolX', 1e-15);
  

% Number of series terms
Kn = 4;
Kx = 3;

% Number of starting values
nstarts = 20;
nstartsnp = 20;

% Read in data
amt2	 = DATA(:,2);
amt3	 = DATA(:,3);
rsrv	 = DATA(:,4);
start	 = DATA(:,5);
ship	 = DATA(:,6);
timing	 = DATA(:,7);
diff1	 = DATA(:,8); % seconds from when amt1 submitted to end of auction
diff2	 = DATA(:,9); % seconds from when amt2 submitted to end of auction
carrcode = DATA(:,11);
iphone4  = DATA(:,12); %dummy for 4 vs 4S
gb       = DATA(:,13); 
black    = DATA(:,14);
   

% Choose time
if time_period==1
    time_flag = ones(length(timing),1)==1; % include both pre and post 
elseif time_period==2
    time_flag = (timing<3); % pre-period only
elseif time_period==3
    time_flag = (timing>=3); % post-period only
end

% Restrict sample
index_use = carrcode==carrier & rsrv>0 & start<=1 & ~isnan(amt3) & time_flag; 
 
% Condition on time
if time_end < 1000
    index_use = index_use & nanmax([diff1 diff2],[],2)<=time_end*3600;
end
 
% run OLS regressions to control for heterogeneity
unique_iphone4 = unique(iphone4);
unique_gb = unique(gb); 
unique_black = unique(black); 
X_temp = zeros(size(iphone4,1),length(unique_black)*length(unique_gb)*length(unique_iphone4)); 
j = 1; 
for b = 1:length(unique_iphone4)
    for c = 1:length(unique_gb)
        for d = 1:length(unique_black)
            X_temp(:,j) = (iphone4==unique_iphone4(b)).*(gb==unique_gb(c)).*(black==unique_black(d)) ;
            j = j+1;
        end
    end
end

index_reg = carrcode<=3 & rsrv>0 & start<=1 & ~isnan(amt3);
X_reg = X_temp(index_reg,:);
col_used = sum(X_reg)>0;
X_reg = X_reg(:,col_used); 
y_temp = log(rsrv(index_reg,1) + ship(index_reg,1));
beta_R = (X_reg'*X_reg)\(X_reg'*y_temp);
 
 
% compute homogenized bids and reserves
R = log(rsrv(index_use) + ship(index_use)) - X_temp(index_use,col_used)*beta_R;
B1 = log(amt2(index_use)+ship(index_use)) - X_temp(index_use,col_used)*beta_R; 
B2 = log(amt3(index_use)+ship(index_use)) - X_temp(index_use,col_used)*beta_R; 
 

% Number of nodes for approximating integrals 
nn = 150;
[nodes, weights] = hermquad(nn);
nodes = nodes(weights>1e-10);
weights = weights(weights>1e-10);
nn = size(nodes,1);

% Sample size
n = size(R,1);

% initialize starting values and constraints
[paramsstart,paramsstart1,A,b,A1,b1,M1,M] = initialize(B2,R, Kn,Kx);

B1(isnan(B1)) = -inf;
B2(isnan(B2)) = -inf;
 
 
fvals = zeros(nstarts,1);
eflags = zeros(nstarts,1);
phat_ps = zeros(size(paramsstart1,1),nstarts); 

% Estimate parametric model
% Try several starting values
for ks = 1:nstarts

    if ks == 1
        paramsstart1temp = paramsstart1+(rand(size(paramsstart1))-0.5)*0.001;
    else
        paramsstart1temp = paramsstart1 + (rand(size(paramsstart1))-0.5).*abs(paramsstart1).*(sum(-A1)') + (rand(size(paramsstart1))-0.5).*(1-sum(-A1)') ;
    end

    [phat_ps(:,ks),fvals(ks,1),eflags(ks,1), ~ , ~, ~, ~] = fmincon(@(params) like_para(params, B1, B2, R, nodes, weights, n, nn),paramsstart1temp, A1, b1,[],[],[],[],[],options);

end
    
% Find optimum
[phat_p, ~] = findmin(phat_ps,fvals,eflags,nstarts);

% Calculate standard errors
hessian_num = calc_hessian(@(params) like_para(params, B1, B2, R, nodes, weights, n, nn),phat_p);
der_p = like_der_par(phat_p, B1, B2, R, nodes, weights, n, nn);
cov_p = inv(hessian_num)*cov(der_p)*inv(hessian_num);
se_p = sqrt(diag(cov_p/n));
    
% Display results
disp('Parametric optimum:') ;
disp(phat_p)
disp(se_p)
    

% Semiparametric model

% Initialize
mu_est = phat_p(6,1);
sig_est = sqrt(2)*phat_p(7,1);  
paramsstart5 = zeros(5+Kn,1);
paramsstart5(1:5,1) = phat_p(1:5,1);
paramsstart5(6:end) = paramsstart(1:Kn,1);

fvals = zeros(nstartsnp,1);
eflags = zeros(nstartsnp,1);
phat_sps = zeros(size(paramsstart5,1),nstartsnp);
lambdas = zeros(nstartsnp,1);
     
% Try several starting values
for ks = 1:nstartsnp

    if ks == 1
        paramsstart1temp = paramsstart5+(rand(size(paramsstart5))-0.5)*0.001;
    else

    paramsstart5(1:5,1) = paramsstart1(1:5,1);
    paramsstart1temp = paramsstart5+(rand(size(paramsstart5))-0.5).*abs(paramsstart5).*(sum(-A)') + (rand(size(paramsstart5))-0.5).*(1-sum(-A)') ;

    paramsstart1temp = fmincon(@(params) make_feasible(params,paramsstart1temp),paramsstart5, A, b, [], [], [], [], @(params) const_semipar(params,Kn,M),options);
    paramsstart1temp = paramsstart1temp+(rand(size(paramsstart1temp))-0.5)*0.001;

    end

    [phat_sps(:,ks),fvals(ks,1),eflags(ks,1), ~ , lambda, ~, ~]  = fmincon(@(params) like_para_semipar(params, B1, B2, R, nodes, weights, n, nn, Kn, mu_est, sig_est),paramsstart1temp, A, b, [], [], [], [], @(params) const_semipar(params,Kn,M),options);        
 
    lambdas(ks,1) = lambda.eqnonlin;

end

% Find optimum
[phat_sp, index] = findmin(phat_sps,fvals,eflags,nstartsnp);

% Calculate standard errors
hessian_num = calc_hessian(@(params) like_para_semipar(params, B1, B2, R, nodes, weights, n, nn, Kn, mu_est, sig_est),phat_sp);
lam = lambdas(index,1);
der_sp = like_der_semipar(phat_sp, B1, B2, R, nodes, weights, n, nn, Kn, mu_est, sig_est);
Gb = zeros(size(paramsstart5,1),1);
G = zeros(size(paramsstart5,1),size(paramsstart5,1));
[Gb(6:5+Kn,1), G(6:5+Kn,6:5+Kn)]  = dconstraint_sp(phat_sp,M,Kn);
A = [ -hessian_num-lam*G  -Gb ; -Gb' 0 ];
B = [ cov(der_sp - lam*ones(n,1)*Gb') zeros(size(paramsstart5,1),1); zeros(1,size(paramsstart5,1)+1)];
cov_all = inv(A)*B*inv(A);
cov_sp = cov_all(1:end-1,1:end-1);
se_sp = sqrt(diag(cov_sp/n));

% Display results
disp('Semiparametric optimum:') ;
disp(phat_sp)
disp(se_sp)


% Nonparametric model

% Initialize
phat_in = phat_sp(1:5);

[mu_v_est,sig_v_est,mu_w_est,sig_w_est,sig_x_est, intargB1_upper,intargB2_upper,~] = initializenp(phat_in,n,nn,nodes,Kn,B1,B2,R);
mu_u2_est = mu_est;
sig_u2_est = sig_est;

paramsstart(end-Kn+1:end,1) = phat_sp(6:end,1);

fvals = zeros(nstartsnp,1);
eflags = zeros(nstartsnp,1);
phat_nps = zeros(size(paramsstart,1),nstartsnp);

lambdas = zeros(5,nstartsnp);

% Try several starting values
for ks = 1:nstartsnp

    if ks == 1
        paramsstart1temp = paramsstart+(rand(size(paramsstart))-0.5)*0.001;
    else
    paramsstart1temp = paramsstart+(rand(size(paramsstart))-0.5)*0.1;
    paramsstart1temp = fmincon(@(params) make_feasible(params,paramsstart1temp),paramsstart, [], [], [], [], [], [], @(params) const_nonpar(params,Kn,Kx,M,M1),options);
    paramsstart1temp = paramsstart1temp+(rand(size(paramsstart1temp))-0.5)*0.001;

    end

    [phat_nps(:,ks),fvals(ks,1),eflags(ks,1), ~ , lambda, ~, ~]  = fmincon(@(params) like_para_nonpar(params, B1, B2, R,  intargB1_upper, intargB2_upper, nodes, weights, n, nn, Kn, Kx, mu_u2_est, sig_u2_est, mu_v_est, sig_v_est, mu_w_est, sig_w_est, sig_x_est ),paramsstart1temp, [], [], [], [], [], [], @(params) const_nonpar(params,Kn,Kx,M,M1),optionsnp);        

    lambdas(:,ks) = lambda.eqnonlin;

end

% Find optimum
[phat_np, index] = findmin(phat_nps,fvals,eflags,nstartsnp);

hessian_num = calc_hessian(@(params) like_para_nonpar(params, B1, B2, R,  intargB1_upper, intargB2_upper, nodes, weights, n, nn, Kn, Kx, mu_u2_est, sig_u2_est, mu_v_est, sig_v_est, mu_w_est, sig_w_est, sig_x_est  ),phat_np);

% Calculate standard errors
lam = lambdas(:,index);
der_np = like_der_nonpar(phat_np, B1, B2, R,  intargB1_upper, intargB2_upper, nodes, weights, n, nn, Kn, Kx, mu_u2_est, sig_u2_est, mu_v_est, sig_v_est, mu_w_est, sig_w_est, sig_x_est);
[Gb, G]  = dconstraint_np(phat_np,Kn,Kx,M,M1);
inpA = G(:,:,1)*lam(1,1)+G(:,:,2)*lam(2,1)+G(:,:,3)*lam(3,1)+G(:,:,4)*lam(4,1)+G(:,:,5)*lam(5,1);
A = [ -hessian_num-inpA  -Gb ; -Gb' zeros(5,5) ];
B = [ cov(der_np - ones(n,1)*(Gb*lam)') zeros(size(paramsstart,1),5); zeros(5,size(paramsstart,1)+5)];
cov_all = inv(A)*B*inv(A);
cov_np = cov_all(1:end-5,1:end-5);
se_np = sqrt(diag(cov_np/n));


% Calculate summary statistics
[sum_stat_np, sum_stat_np_med] = sumstats(phat_np, Kn,Kx, M1, mu_v_est, sig_v_est, mu_w_est, sig_w_est, sig_x_est);

[se_sum_stat_np, se_sum_stat_np_med] = sumstats_se(phat_np, Kn, Kx, M1, mu_v_est, sig_v_est, mu_w_est, sig_w_est, sig_x_est,cov_np,n);

 
out_vec = [phat_p;phat_sp;phat_np;sum_stat_np;n;se_p;se_sp; se_sum_stat_np;se_np;reshape(cov_p,size(cov_p,1)^2,1);reshape(cov_sp,size(cov_sp,1)^2,1);reshape(cov_np,size(cov_np,1)^2,1);sum_stat_np_med;se_sum_stat_np_med];
 
     


end



% initialize starting values and constraints
function [paramsstart,paramsstart1,A,b,A1,b1,M1,M] = initialize(B2,R,Kn,Kx)

% Matrix used for quadratic constraint that density integrates to 1
M = intoneconst(Kn);
% Matrix used for quadratic constraint that the mean is 0
M1 = intoneconst(Kn+1);
M1 = M1(2:end,1:Kn);
% Constraint matrix: Nonnegative standard deviations
A = zeros(3,5+Kn); A(1,3) = -1; A(2,4) = -1; A(3,5) = -1;
b = -0*ones(3,1);
% Constraint matrix: Nonnegative standard deviations for first step prametric estimator
A1 = zeros(4,7); A1(1,3) = -1; A1(2,4) = -1; A1(3,5) = -1; A1(4,7) = -1;
b1 = -0*ones(4,1);
% Starting values
paramsstart = [sqrt(1/sqrt(pi));zeros(Kn-1,1);sqrt(1/sqrt(pi));zeros(Kn-1,1);sqrt(1/sqrt(pi));zeros(Kx-1,1);sqrt(1/sqrt(pi));zeros(Kn-1,1)];

paramsstart1 = zeros(7,1);
paramsstart1(1,1) = mean(B2(~isnan(B2)));
paramsstart1(2,1) = mean(R);
paramsstart1(3,1) = sqrt(var(B2(~isnan(B2))));
paramsstart1(4,1) = sqrt(var(R));
paramsstart1(5,1) = 0.5;
paramsstart1(6,1) = mean(B2(~isnan(B2)));
paramsstart1(7,1) = sqrt(var(B2(~isnan(B2))));

end


function [phat_3, ind_out ] = findmin(phat_3s,fvals,eflags,nstarts)

[~ ,indf] = sort( fvals(:,1) );

minf = 0;
j = 1;
while minf == 0

    if eflags(indf(j,1),1)>0 || eflags(indf(j,1),1)==-3 || eflags(indf(j,1),1)==-100 || eflags(indf(j,1),1)==-101|| eflags(indf(j,1),1)==-102 || eflags(indf(j,1),1)==-103 || j == nstarts
        phat_3 = phat_3s(:,indf(j,1));
        minf = 1;
    else 
        j=j+1;
    end

end
        
ind_out = indf(j,1);

disp(fvals)
disp(eflags)
disp(phat_3s)

end

function [mu_v_est,sig_v_est,mu_w_est,sig_w_est,sig_x_est ,intargB1_upper,intargB2_upper,intargR] = initializenp(phat_3,n,nn,nodes,Kn,B1,B2,R)
     
mu_v_est = phat_3(1,1);
sig_v_est = sqrt(2)*phat_3(3,1);  
mu_w_est = phat_3(2,1);
sig_w_est = sqrt(2)*phat_3(4,1);  
sig_x_est = sqrt(2)*phat_3(5,1);  

nodes = sig_x_est*nodes;

intargB1_upper = zeros(n,nn,(Kn-1)^2+1);
tempB1 = ( ( B1*ones(1,nn) - ones(n,1)*nodes'*1 ) - mu_v_est)/sig_v_est;
for j = 0:(Kn-1)^2
    intargB1_upper(:,:,j+1) = intxex_upper(j,tempB1);
end

intargB2_upper = zeros(n,nn,(Kn-1)^2+1);
tempB2 = ( ( B2*ones(1,nn) - ones(n,1)*nodes'*1 ) - mu_v_est)/sig_v_est;
for j = 0:(Kn-1)^2
    intargB2_upper(:,:,j+1) = intxex_upper(j,tempB2);
end


intargR = zeros(n,nn,(Kn-1)^2+1);
tempR = ( ( R*ones(1,nn) - ones(n,1)*nodes'*1 ) - mu_v_est)/sig_v_est;
for j = 0:(Kn-1)^2
    intargR(:,:,j+1) = intxex(j,tempR);
end
        
end


% Likelihood for the parametric model
function f = like_para(params, B1, B2, R, nodes, weights, n, nn )

mu_v = params(1,1);
mu_w = params(2,1);

sig_v = params(3,1);
sig_w = params(4,1);
sig_x = params(5,1);
 
mu_u2 = params(6,1);
sig_u2 = params(7,1);

nodes = sig_x*nodes;
 
% Marginal density of u2 assuming its normal
du2 =  densu2(B2*ones(1,nn) - ones(n,1)*nodes'*1, [mu_u2;sig_u2]);

densu1gu2 = 2*(distv_upper( B1*ones(1,nn) - ones(n,1)*nodes'*1, [mu_v;sig_v]) ).*densv( B1*ones(1,nn) - ones(n,1)*nodes'*1, [mu_v;sig_v]).*(distv_upper( B2*ones(1,nn) - ones(n,1)*nodes'*1, [mu_v;sig_v]) + 0.1^30).^(-2);

dens = sig_x*du2.*densu1gu2.*densw(R*ones(1,nn) - ones(n,1)*nodes'*1, [mu_w;sig_w])*(densx(nodes,sig_x).*exp((nodes/sig_x).^2).*weights);

dens = dens + (dens==0)*(0.1^50);

f = -mean(log(dens) );

end


function der = like_der_par(params, B1, B2, R, nodes, weights, n, nn )
% This is a function to calculate the derivatives of the parametric likelihood
% function given Z_t, i.e., for an individual auction

%%% Output 
% der: a matrix of derivatives, in the order of parameters (dim:
% n by dim(params) )

mu_v = params(1,1);
mu_w = params(2,1);

sig_v = params(3,1);
sig_w = params(4,1);
sig_x = params(5,1);
 
mu_u2 = params(6,1);
sig_u2 = params(7,1);

nodes = sig_x*nodes;

% Calculate the integral within the likelihood for a given point
% Marginal density of u2 assuming its normal
du2 =  densu2(B2*ones(1,nn) - ones(n,1)*nodes'*1, [mu_u2;sig_u2]);

% Density of u1 given u2
distv_upper_b1 = (distv_upper( B1*ones(1,nn) - ones(n,1)*nodes'*1, [mu_v;sig_v]) );
distv_upper_b2 = (distv_upper( B2*ones(1,nn) - ones(n,1)*nodes'*1, [mu_v;sig_v]) );
densv_b1 = densv( B1*ones(1,nn) - ones(n,1)*nodes'*1, [mu_v;sig_v]);

densu1gu2 = 2*distv_upper_b1.*densv_b1.*( distv_upper_b2 + 0.1^30).^(-2);

% Density of x
dx = densx(nodes,sig_x);

% Density of w
dw = densw(R*ones(1,nn) - ones(n,1)*nodes'*1, [mu_w;sig_w]);

% Value of the integral for all auctions
dens = sig_x*du2.*densu1gu2.*dw*(dx.*exp((nodes/sig_x).^2).*weights);
dens = dens + (dens==0)*(0.1^50);

% Calculate derivatives of densities with respect to parameters
ddenswdmu_w = dnormpdfdmu(R*ones(1,nn) - ones(n,1)*nodes'*1, mu_w, sig_w);
ddenswdsig_w = dnormpdfdsig(R*ones(1,nn) - ones(n,1)*nodes'*1, mu_w, sig_w);
ddu2dmu_u2 = dnormpdfdmu(B2*ones(1,nn) - ones(n,1)*nodes'*1, mu_u2, sig_u2);
ddu2dsig_u2 = dnormpdfdsig(B2*ones(1,nn) - ones(n,1)*nodes'*1, mu_u2, sig_u2);
ddxdsig_x = dnormpdfdsig(nodes, 0, sig_x);

densv_b2 = densv( B2*ones(1,nn) - ones(n,1)*nodes'*1, [mu_v;sig_v]);

ddensv_b1_mu = dnormpdfdmu(B1*ones(1,nn) - ones(n,1)*nodes'*1,mu_v,sig_v);
ddensv_b1_sigma = dnormpdfdsig(B1*ones(1,nn) - ones(n,1)*nodes'*1,mu_v,sig_v);

ddensu1gu2dsig_v = 2*(  (  ((B1*ones(1,nn) - ones(n,1)*nodes'*1 - mu_v)/sig_v.*densv_b1).*densv_b1 + (distv_upper_b1).*ddensv_b1_sigma ).*( distv_upper_b2 + 0.1^30).^(2) - (2)*distv_upper_b1.*densv_b1.*( distv_upper_b2  + 0.1^30 ).*( (B2*ones(1,nn) - ones(n,1)*nodes'*1 - mu_v)/sig_v.*densv_b2) ) .*( distv_upper_b2  + 0.1^30 ).^(-4);
ddensu1gu2dmu_v = 2*(  (  (densv_b1).*densv_b1 + ((distv_upper_b1)).*ddensv_b1_mu ).*( distv_upper_b2 + 0.1^30).^(2) - (2)*distv_upper_b1.*densv_b1.*( distv_upper_b2  + 0.1^30 ).*(densv_b2) ) .*( distv_upper_b2  + 0.1^30 ).^(-4);

% Calculate the individual vectors of derivatives
dldmu_v = 1./dens.*(sig_x*ddensu1gu2dmu_v.*du2.*dw*(dx.*exp((nodes/sig_x).^2).*weights));
dldmu_w = 1./dens.*(sig_x*densu1gu2.*du2.*ddenswdmu_w*(dx.*exp((nodes/sig_x).^2).*weights));
dldsig_v = 1./dens.*(sig_x*ddensu1gu2dsig_v.*du2.*dw*(dx.*exp((nodes/sig_x).^2).*weights));
dldsig_w = 1./dens.*(sig_x*densu1gu2.*du2.*ddenswdsig_w*(dx.*exp((nodes/sig_x).^2).*weights));
dldsig_x = 1./dens.*(sig_x*densu1gu2.*du2.*dw*(ddxdsig_x.*exp((nodes/sig_x).^2).*weights));
dldmu_u2 = 1./dens.*(sig_x*densu1gu2.*dw.*ddu2dmu_u2*(dx.*exp((nodes/sig_x).^2).*weights));
dldsig_u2 = 1./dens.*(sig_x*densu1gu2.*dw.*ddu2dsig_u2*(dx.*exp((nodes/sig_x).^2).*weights));


% Combine into a matrix output
der = [dldmu_v, dldmu_w, dldsig_v, dldsig_w, dldsig_x, dldmu_u2, dldsig_u2];

end 
    

% Likelihood for the semiparametric model
function f = like_para_semipar(params, B1, B2, R, nodes, weights, n, nn, Kn, mu_est, sig_est)

mu_v = params(1,1);
mu_w = params(2,1);

sig_v = params(3,1);
sig_w = params(4,1);
sig_x = params(5,1);

au2 = params(6:5+Kn,1);

nodes = sig_x*nodes;

% Marginal density of u2 estimated nonparametrically
du2 = npdens(B2*ones(1,nn) - ones(n,1)*nodes'*1,au2,Kn,mu_est,sig_est);
 
% Density of u1|u2. Make sure it is not inf
densu1gu2 = 2*(distv_upper( B1*ones(1,nn) - ones(n,1)*nodes'*1, [mu_v;sig_v] ) ).*densv( B1*ones(1,nn) - ones(n,1)*nodes'*1, [mu_v;sig_v]).*(distv_upper( B2*ones(1,nn) - ones(n,1)*nodes'*1, [mu_v;sig_v]) + 0.1^30).^(-2);

dens = sig_x*du2.*densu1gu2.*densw(R*ones(1,nn) - ones(n,1)*nodes'*1, [mu_w;sig_w])*(densx(nodes,sig_x).*exp((nodes/sig_x).^2).*weights);

dens = dens + (dens==0)*(0.1^50);

f = -mean(log(dens));

end



function der = like_der_semipar(params, B1, B2, R, nodes, weights, n, nn, Kn, mu_est, sig_est)
% This is a function to calculate the derivatives of the semi-parametric likelihood
% function given Z_t, i.e., for an individual auction
 
 
%%% Output 
% der: a matrix of derivatives, in the order of parameters (dim:
% n by dim(params) )

mu_v = params(1,1);
mu_w = params(2,1);

sig_v = params(3,1);
sig_w = params(4,1);
sig_x = params(5,1);
 
au2 = params(6:5+Kn,1);

nodes = sig_x*nodes;

 
% Marginal density of u2 estimated nonparametrically
du2 = npdens(B2*ones(1,nn) - ones(n,1)*nodes'*1,au2,Kn,mu_est,sig_est);
 
% Density of u1 given u2
distv_upper_b1 = (distv_upper( B1*ones(1,nn) - ones(n,1)*nodes'*1, [mu_v;sig_v]) );
distv_upper_b2 = (distv_upper( B2*ones(1,nn) - ones(n,1)*nodes'*1, [mu_v;sig_v]) );
densv_b1 = densv( B1*ones(1,nn) - ones(n,1)*nodes'*1, [mu_v;sig_v]);

densu1gu2 = 2*distv_upper_b1.*densv_b1.*( distv_upper_b2 + 0.1^30).^(-2);

% Density of dx
dx = densx(nodes,sig_x);

% Density of dw
dw = densw(R*ones(1,nn) - ones(n,1)*nodes'*1, [mu_w;sig_w]);

% Value of the integral for all auctions
dens = sig_x*du2.*densu1gu2.*dw*(dx.*exp((nodes/sig_x).^2).*weights);
dens = dens + (dens==0)*(0.1^50);

% Calculate the derivatives of the likelihood function
% Calculate derivatives within the expressions
ddenswdmu_w = dnormpdfdmu(R*ones(1,nn) - ones(n,1)*nodes'*1, mu_w, sig_w);
ddenswdsig_w = dnormpdfdsig(R*ones(1,nn) - ones(n,1)*nodes'*1, mu_w, sig_w);
ddxdsig_x = dnormpdfdsig(nodes, 0, sig_x);
ddensu2dau2 = dnpdensda(B2*ones(1,nn) - ones(n,1)*nodes'*1,au2,Kn,mu_est,sig_est);
 
densv_b2 = densv( B2*ones(1,nn) - ones(n,1)*nodes'*1, [mu_v;sig_v]);

ddensv_b1_mu = dnormpdfdmu(B1*ones(1,nn) - ones(n,1)*nodes'*1,mu_v,sig_v);
ddensv_b1_sigma = dnormpdfdsig(B1*ones(1,nn) - ones(n,1)*nodes'*1,mu_v,sig_v);

ddensu1gu2dsig_v = 2*(  (((B1*ones(1,nn) - ones(n,1)*nodes'*1 - mu_v)/sig_v.*densv_b1).*densv_b1 + ((distv_upper_b1)).*ddensv_b1_sigma ).*( distv_upper_b2 + 0.1^30).^(2) - (2)*distv_upper_b1.*densv_b1.*( distv_upper_b2  + 0.1^30 ).*( (B2*ones(1,nn) - ones(n,1)*nodes'*1 - mu_v)/sig_v.*densv_b2) ) .*( distv_upper_b2  + 0.1^30 ).^(-4);
ddensu1gu2dmu_v = 2*(  ( (densv_b1).*densv_b1 + ((distv_upper_b1)).*ddensv_b1_mu ).*( distv_upper_b2 + 0.1^30).^(2) - (2)*distv_upper_b1.*densv_b1.*( distv_upper_b2  + 0.1^30 ).*(densv_b2) ) .*( distv_upper_b2  + 0.1^30 ).^(-4);


% Calculate the individual vectors of derivatives
dldmu_w = 1./dens.*(sig_x*densu1gu2.*du2.*ddenswdmu_w*(dx.*exp((nodes/sig_x).^2).*weights));
dldsig_w = 1./dens.*(sig_x*densu1gu2.*du2.*ddenswdsig_w*(dx.*exp((nodes/sig_x).^2).*weights));
dldsig_x = 1./dens.*(sig_x*densu1gu2.*du2.*dw*(ddxdsig_x.*exp((nodes/sig_x).^2).*weights));
dldmu_v = 1./dens.*(sig_x*ddensu1gu2dmu_v.*du2.*dw*(dx.*exp((nodes/sig_x).^2).*weights));
dldsig_v = 1./dens.*(sig_x*ddensu1gu2dsig_v.*du2.*dw*(dx.*exp((nodes/sig_x).^2).*weights));

dldau2 = zeros(n, Kn);

for j = 1:Kn
    dldau2(:,j) = 1./dens.*(sig_x*densu1gu2.*dw.*ddensu2dau2(:,:,j)*(dx.*exp((nodes/sig_x).^2).*weights));
end

% Combine into a matrix output
der = [dldmu_v, dldmu_w, dldsig_v, dldsig_w, dldsig_x, dldau2];


end



% Likelihood for nonparametric model
function f = like_para_nonpar(params, B1, B2, R,  intargB1_uppper, intargB2_uppper, nodes, weights, n, nn, Kn, Kx, mu_u2_est, sig_u2_est, mu_v_est, sig_v_est, mu_w_est, sig_w_est, sig_x_est)

av = params(1:Kn,1);
aw = params(Kn+1:2*Kn,1);
ax = params(2*Kn+1:2*Kn+Kx,1);
au2 = params(2*Kn+Kx+1:3*Kn+Kx,1);
 

nodes = sig_x_est*nodes;

% Marginal density of u2 estimated nonparametrically
densu2n = npdens(B2*ones(1,nn) - ones(n,1)*nodes'*1,au2,Kn,mu_u2_est,sig_u2_est);

distvn1_upper = distvnp(intargB1_uppper,av);
distvn2_upper = distvnp(intargB2_uppper,av);

densvn1 = densvnp( B1*ones(1,nn) - ones(n,1)*nodes'*1,av,Kn,mu_v_est,sig_v_est);
denswn = npdens(R*ones(1,nn) - ones(n,1)*nodes'*1, aw,Kn,mu_w_est,sig_w_est);
densxn = npdens(nodes, ax, Kx, 0, sig_x_est);

% Density of u1|u2. Make sure it is not inf
densu1gu2 = 2*(distvn1_upper).*densvn1.*( distvn2_upper  + 0.1^30 ).^(-2);

dens = sig_x_est*densu2n.*densu1gu2.*denswn*(densxn.*exp((nodes/sig_x_est).^2).*weights);

dens = dens + (dens==0)*(0.1^50);

f = -mean(log(dens));
 

end

% Derivatives of likelihood with all data: Method 7
function der = like_der_nonpar(params, B1, B2, R,  intargB1_upper, intargB2_upper, nodes, weights, n, nn, Kn, Kx, mu_u2_est, sig_u2_est, mu_v_est, sig_v_est, mu_w_est, sig_w_est, sig_x_est)
% This is a function to calculate the derivatives of the nonparametric likelihood
% function given Z_t, i.e., for an individual auction

%%% Output 
% der: a matrix of derivatives, in the order of parameters (dim:
% n by dim(params) )


av = params(1:Kn,1);
aw = params(Kn+1:2*Kn,1);
ax = params(2*Kn+1:2*Kn+Kx,1);
au2 = params(2*Kn+Kx+1:3*Kn+Kx,1);
  
nodes = sig_x_est*nodes;

% Marginal density of u2 estimated nonparametrically
densu2n = npdens(B2*ones(1,nn) - ones(n,1)*nodes'*1,au2,Kn,mu_u2_est,sig_u2_est);

distvn1_upper = distvnp(intargB1_upper,av);
distvn2_upper = distvnp(intargB2_upper,av);

densvn1 = densvnp( B1*ones(1,nn) - ones(n,1)*nodes'*1,av,Kn,mu_v_est,sig_v_est);
denswn = npdens(R*ones(1,nn) - ones(n,1)*nodes'*1, aw,Kn,mu_w_est,sig_w_est);
densxn = npdens(nodes, ax, Kx, 0, sig_x_est);

% Density of u1|u2. Make sure it is not inf
densu1gu2 = 2*(distvn1_upper).*densvn1.*( distvn2_upper  + 0.1^30 ).^(-2);

dens = sig_x_est*densu2n.*densu1gu2.*denswn*(densxn.*exp((nodes/sig_x_est).^2).*weights);

% Calculate the derivatives of the likelihood function
% Calculate derivatives within the expressions
ddenswdaw = dnpdensda(R*ones(1,nn) - ones(n,1)*nodes'*1, aw, Kn, mu_w_est, sig_w_est);
ddensu2dau2 = dnpdensda(B2*ones(1,nn) - ones(n,1)*nodes'*1, au2, Kn, mu_u2_est, sig_u2_est);
ddensxdax = dnpdensda(nodes, ax, Kx, 0, sig_x_est);
 

ddistvn1_upper = ddistvnp(intargB1_upper,av);
ddistvn2_upper = ddistvnp(intargB2_upper,av);
ddensvda = dnpdensda(B1*ones(1,nn) - ones(n,1)*nodes'*1, av, Kn, mu_v_est, sig_v_est);

ddensu1gu2da = zeros([size(ddistvn1_upper), Kn]);
for j = 1:Kn
    ddensu1gu2da(:,:,j) =  2*(  ( ddistvn1_upper(:,:,j).*densvn1 + ((distvn1_upper)).*ddensvda(:,:,j) ).*( distvn2_upper  + 0.1^30 ).^(2) - (2)*((distvn1_upper)).*densvn1.*( distvn2_upper  + 0.1^30 ).*ddistvn2_upper(:,:,j) ) .*( distvn2_upper  + 0.1^30 ).^(-4);
end

 
% Calculate the individual vectors of derivatives
dldav = zeros([n,Kn]);
dldaw = zeros([n,Kn]);
dldax = zeros([n,Kx]);
dldau2 = zeros([n,Kn]);


for j = 1:Kn
    dldav(:,j) = 1./dens.*(sig_x_est*densu2n.*ddensu1gu2da(:,:,j).*denswn*(densxn.*exp((nodes/sig_x_est).^2).*weights));
    dldaw(:,j) = 1./dens.*(sig_x_est*densu2n.*densu1gu2.*ddenswdaw(:,:,j)*(densxn.*exp((nodes/sig_x_est).^2).*weights));
    dldau2(:,j) = 1./dens.*(sig_x_est*ddensu2dau2(:,:,j).*densu1gu2.*denswn*(densxn.*exp((nodes/sig_x_est).^2).*weights));
end

for j = 1:Kx
    dldax(:,j) = 1./dens.*(sig_x_est*densu2n.*densu1gu2.*denswn*(ddensxdax(:,j).*exp((nodes/sig_x_est).^2).*weights));
end


% Combine into a matrix output
der = [dldav, dldaw, dldax, dldau2];


end




function f = densv(x,parms)

mu = parms(1,1);
sig = parms(2,1);

f = (1/sig)*normpdf( (x - mu)/sig,0,1);

end

function f = densvnp(x,ax,Kn,mu,sig)

fx = zeros(size(x));

y = (x-mu)/sig;

for j=1:Kn
    fx = fx + (y).^(j-1)*ax(j,1);
end

f = (1/sig)*(fx.^2).*exp(-( y ).^2 );

end

function f = densx(x,sig_x)

f = normpdf(x,0,sig_x);

end

function f = densw(x,parms)

mu = parms(1,1);
sig = parms(2,1);

f = (1/sig)*normpdf( (x - mu)/sig,0,1);

end

function f = densu2(x,parms)

mu = parms(1,1);
sig = parms(2,1);

f = (1/sig)*normpdf( (x - mu)/sig,0,1);

end
 


function f = distv_upper(x,parms)

mu = parms(1,1);
sig = parms(2,1);

f = normcdf( (x - mu)/sig,0,1,'upper');

end


function f = distvnp(intarg,ax)

fx = zeros(size(intarg(:,:,1)));
 
for j=1:size(ax,1)
    for k=1:size(ax,1)
        fx = fx + ax(j,1)*ax(k,1)*intarg(:,:,(j-1)+(k-1)+1);%intxex((j-1)+(k-1),y);
    end
end

fx = fx.*(fx>=0) + zeros(size(fx)).*(fx<0);

f = fx.*(fx<=1) + ones(size(fx)).*(fx>1);
 
end


function dfx = ddistvnp(intarg,ax)

fx = zeros(size(intarg(:,:,1)));
 
for j=1:size(ax,1)
    for k=1:size(ax,1)
        fx = fx + ax(j,1)*ax(k,1)*intarg(:,:,(j-1)+(k-1)+1);%intxex((j-1)+(k-1),y);
    end
end

fx = fx.*(fx>=0) + zeros(size(fx)).*(fx<0);

f = fx.*(fx<=1) + ones(size(fx)).*(fx>1);
 

dfx = zeros(size(intarg(:,:,1),1),size(intarg(:,:,1),2),size(ax,1));

for j=1:size(ax,1)
    for k=1:size(ax,1)
        dfx(:,:,j) = dfx(:,:,j) + 2*ax(k,1)*intarg(:,:,(j-1)+(k-1)+1);%intxex((j-1)+(k-1),y);
    end
    
    dfx(:,:,j) = dfx(:,:,j).*((f>0).*(f<1));

end

end

% Calculates int^y_{-inf} x^p e^{-x^2}
function f = intxex(p,y)

if p == 0
	f = sqrt(pi)*normcdf(y,0,sqrt(1/2));
elseif p == 1
	f = (-1/2)*exp(-( y ).^2 );
else
    f = (p-1)/2*intxex(p-2,y) - (1/2)*y.^(p-1).*exp(-( y ).^2 );
end

end


% Calculates int^y_{-inf} x^p e^{-x^2}
function f = intxex_upper(p,y)

if p == 0
	f = sqrt(pi)*normcdf(y,0,sqrt(1/2),'upper');
elseif p == 1
	f = (1/2)*exp(-( y ).^2 );
else
    f = (p-1)/2*intxex_upper(p-2,y) + (1/2)*y.^(p-1).*exp(-( y ).^2 );
end

end


function f = npdens(x,ax,Kn,mu,sig)

fx = zeros(size(x));

y = (x-mu)/sig; 

for j=1:Kn
    fx = fx + (y).^(j-1)*ax(j,1);
end

f = (1/sig)*(fx.^2).*exp(-( y ).^2 );

end


function f = npdist(x,ax,Kn,mu,sig)
 
    % Calculates int^y_{-inf} x^p e^{-x^2}
   
    f = zeros(size(x));
    for j = 1:Kn
        for k = 1:Kn
          f = f + ax(j,1)*ax(k,1)*intxex((j-1)+(k-1),(x-mu)/sig);
        end
    end
    
end


function f = dnpdist(x,ax,Kn,mu,sig)
 
    % Calculates int^y_{-inf} x^p e^{-x^2}
   
    f = zeros(size(x,1),Kn);
    for j = 1:Kn
        
        for k = 1:Kn
          f(:,j) = f(:,j) + 2*ax(k,1)*intxex((j-1)+(k-1),(x-mu)/sig);
        end
        
    end
    
end



% Functions needed for calculating the derivatives

function der = dnormpdfdmu(x,mu,sig) 
% A function for calculating the derivative of normal PDF of X with respect to
% mu, where X~N(mu,sigma^2) at point x

    der = normpdf(x,mu,sig).*(x-mu)/sig^2;
end

function der = dnormpdfdsig(x,mu,sig)
% A function for calculating the derivative of normal PDF of X with respect to
% sigma, where X~N(mu,sigma^2) at point x

    der = normpdf(x,mu,sig)/sig.*( -1 + ((x-mu)/sig).^2 );
end
 
            
function der = dnpdensda(x,ax,Kn,mu,sig)
% A function for calculating the derivative of the non-parametric densities
% with respect to ax's
%%% Output
% der: derivatives with respect to ax's (a size(x)*dim(a) 3D matrix)

    y = (x-mu)/sig;
    
    fx = zeros(size(x));
    
    der = zeros([size(x),size(ax)]);
    
    for j = 1:Kn
        fx = fx + (y).^(j-1)*ax(j,1);
    end
    
    for l = 1:Kn
        der(:,:,l) = 2/sig*exp(-y.^2).*fx.*y.^(l-1);
    end
end


function [c, ceq] = const_semipar(params,Kn,M)

au2 = params(6:5+Kn,1);
ceq = constdens1(au2,M);
c = [];

end

function [c, ceq] = const_nonpar(params,Kn,Kx,M,M1)

av = params(1:Kn,1);
aw = params(Kn+1:2*Kn,1);
ax = params(2*Kn+1:2*Kn+Kx,1);
au2 = params(2*Kn+Kx+1:3*Kn+Kx,1);
 
ceq = [constdens1(av,M);constdens1(aw,M);constdens1(ax,M);constdens1(au2,M);ax'*M1(1:size(ax,1),1:size(ax,1))*ax];
c = [];

end


function ceq = constdens1(ax,M)

ceq = 1 - ax'*M(1:size(ax,1),1:size(ax,1))*ax;

end


function  [deq, d2eq]  = dconstraint_sp(params,M,Kn)

au2 = params(6:5+Kn,1);

deq = -2*M(1:size(au2,1),1:size(au2,1))*au2;

d2eq = -2*M(1:size(au2,1),1:size(au2,1));

end



function [deq, d2eq]  = dconstraint_np(params,Kn,Kx,M,M1)

av = params(1:Kn,1);
aw = params(Kn+1:2*Kn,1);
ax = params(2*Kn+1:2*Kn+Kx,1);
au2 = params(2*Kn+Kx+1:3*Kn+Kx,1);

deq = zeros(3*Kn+Kx,5);

d2eq = zeros(3*Kn+Kx,3*Kn+Kx,5);


deq(1:Kn,1) = -2*M(1:size(av,1),1:size(av,1))*av;
deq(Kn+1:2*Kn,2) = -2*M(1:size(aw,1),1:size(aw,1))*aw;
deq(2*Kn+1:2*Kn+Kx,3) = -2*M(1:size(ax,1),1:size(ax,1))*ax;
deq(2*Kn+Kx+1:3*Kn+Kx,4) = -2*M(1:size(au2,1),1:size(au2,1))*au2;

deq(2*Kn+1:2*Kn+Kx,5) = 2*M1(1:size(ax,1),1:size(ax,1))*ax;


d2eq(1:Kn,1:Kn,1) = -2*M(1:size(av,1),1:size(av,1));
d2eq(Kn+1:2*Kn,Kn+1:2*Kn,2) = -2*M(1:size(aw,1),1:size(aw,1));
d2eq(2*Kn+1:2*Kn+Kx,2*Kn+1:2*Kn+Kx,3) = -2*M(1:size(ax,1),1:size(ax,1));
d2eq(2*Kn+Kx+1:3*Kn+Kx,2*Kn+Kx+1:3*Kn+Kx,4) = -2*M(1:size(au2,1),1:size(au2,1));

d2eq(2*Kn+1:2*Kn+Kx,2*Kn+1:2*Kn+Kx,5) = 2*M1(1:size(ax,1),1:size(ax,1));


end



function M = intoneconst(J)

M = zeros(J,J);
for j=1:J
    for k=1:J
        if mod(j+k,2)==0
            M(j,k) = ((1/sqrt(2))^(j+k-2) )*sqrt(pi)*(dfact(j+k-2)) ;
        end
    end
end
    
end


function f = dfact(k)

k2 = k/2;
f=1;
for i=1:k2
	f = f*(2*i-1); 
end

end


function [phat, phat_med] = sumstats(phatnp, Kn, Kx, M1, mu_v_est, sig_v_est, mu_w_est, sig_w_est, sig_x_est)

% Calculate summary stats
av = phatnp(1:Kn,1);
aw = phatnp(Kn+1:2*Kn,1);
ax = phatnp(2*Kn+1:2*Kn+Kx,1);

phat = zeros(5,1);

phat(1,1) = sig_v_est*av'*M1*av+mu_v_est;
phat(2,1) = sig_w_est*aw'*M1*aw+mu_w_est;

M2 = intoneconst(Kn+2);
M2 = M2(3:end,1:Kn);

phat(3,1) = sqrt(sig_v_est^2*av'*M2*av + mu_v_est^2 + 2*mu_v_est*sig_v_est*av'*M1*av - phat(1,1)^2);
phat(4,1) = sqrt(sig_w_est^2*aw'*M2*aw + mu_w_est^2 + 2*mu_w_est*sig_w_est*aw'*M1*aw - phat(2,1)^2);


phat(5,1) = sqrt(sig_x_est^2*ax'*M2(1:size(ax,1),1:size(ax,1))*ax - 0);


phat_med = zeros(2,1);

phat_med(1,1) = fsolve(@(x) npdist(x,av,Kn,phat(1,1),phat(3,1))-0.5,0);
phat_med(2,1) = fsolve(@(x) npdist(x,aw,Kn,phat(2,1),phat(4,1))-0.5,0);

 
 
end
         


function [se_phat, se_med] = sumstats_se(phatnp, Kn, Kx, M1, mu_v_est, sig_v_est, mu_w_est, sig_w_est, sig_x_est,covm,n)

% Calculate summary stats
av = phatnp(1:Kn,1);
aw = phatnp(Kn+1:2*Kn,1);
ax = phatnp(2*Kn+1:2*Kn+Kx,1);

phat = zeros(5,1);
se_phat = zeros(5,1);

phat(1,1) = sig_v_est*av'*M1*av+mu_v_est;
phat(2,1) = sig_w_est*aw'*M1*aw+mu_w_est;
M2 = intoneconst(Kn+2);
M2 = M2(3:end,1:Kn);
phat(3,1) = sqrt(sig_v_est^2*av'*M2*av + mu_v_est^2 + 2*mu_v_est*sig_v_est*av'*M1*av - phat(1,1)^2);
phat(4,1) = sqrt(sig_w_est^2*aw'*M2*aw + mu_w_est^2 + 2*mu_w_est*sig_w_est*aw'*M1*aw - phat(2,1)^2);

g1 = 2*sig_v_est*M1*av;
se_phat(1,1) = sqrt(g1'*covm(1:Kn,1:Kn)*g1/n);

g2 = 2*sig_v_est*M1*aw;
se_phat(2,1) = sqrt(g2'*covm(Kn+1:2*Kn,Kn+1:2*Kn)*g2/n);

M2 = intoneconst(Kn+2);
M2 = M2(3:end,1:Kn);

g3 = 0.5*( 2*sig_v_est^2*M2*av + 4*mu_v_est*sig_v_est*M1*av - 2*phat(1,1)*g1  )/sqrt(sig_v_est^2*av'*M2*av + mu_v_est^2 + 2*mu_v_est*sig_v_est*av'*M1*av - phat(1,1)^2);
se_phat(3,1) = sqrt(g3'*covm(1:Kn,1:Kn)*g3/n);

g4 = 0.5*( 2*sig_w_est^2*M2*aw + 4*mu_w_est*sig_w_est*M1*aw - 2*phat(2,1)*g2  )/sqrt(sig_w_est^2*aw'*M2*aw + mu_w_est^2 + 2*mu_w_est*sig_w_est*aw'*M1*aw - phat(2,1)^2);
se_phat(4,1) = sqrt(g4'*covm(Kn+1:2*Kn,Kn+1:2*Kn)*g4/n);

g5 = sig_x_est^2*M2(1:size(ax,1),1:size(ax,1))*ax/sqrt(sig_x_est^2*ax'*M2(1:size(ax,1),1:size(ax,1))*ax);
se_phat(5,1) = sqrt(g5'*covm(2*Kn+1:2*Kn+Kx,2*Kn+1:2*Kn+Kx)*g5/n);


phat_med = zeros(2,1);
se_med = zeros(2,1);

phat_med(1,1) = fsolve(@(x) npdist(x,av,Kn,phat(1,1),phat(3,1))-0.5,0);
phat_med(2,1) = fsolve(@(x) npdist(x,aw,Kn,phat(2,1),phat(4,1))-0.5,0);


g_med_1 = dnpdist(phat_med(1,1),av,Kn,phat(1,1),phat(3,1))'.^(-1);
se_med(1,1) = sqrt(g_med_1'*covm(1:Kn,1:Kn)*g_med_1/n);

g_med_2 = dnpdist(phat_med(2,1),aw,Kn,phat(2,1),phat(4,1))'.^(-1);
se_med(2,1) = sqrt(g_med_2'*covm(Kn+1:2*Kn,Kn+1:2*Kn)*g_med_2/n);


end
         


function f = make_feasible(params,paramsstart)

f = (params-paramsstart)'*(params-paramsstart);

end


function [X, W] = hermquad(N)
%
% [X W] = HERMQUAD(N)
%
% Find the Gauss-Hermite abscissae and weights.
%
% Arguments:
%  N - The number of abscissae and weights to return.
%
% Return Values:
%  X - A column vector containing the abscissae.
%  W - A column vector containing the corresponding weights.
%
% Gauss-Hermite quadrature approximates definite integrals of the form
%
%     \int^{-\infty}_{\infty} dx W(x) f(x)
%
% where
%
%     W(x) = \exp( - x^2 )
%
% with the sum
%
%     \sum_{n=1}^{N} w_{n} f(x_{n}).
%
% This function returns the set of abscissae and weights
%
%     {x_{n}, w_{n}}^{N}_{n=1}
%
% for performing this calculation given N, the number of abscissae.
% These abscissae correspond to the zeros of the Nth Hermite
% polynomial.  It can be shown that such integration is exact when f(x)
% is a polynomial of maximum order 2N-1.
%
% The procedure in this calculation is taken more or less directly from
%
% @BOOK{ press-etal-1992a,
%	 AUTHOR   = { Press, William  H.   and 
%                     Flannery, Brian  P.  and
%                     Teukolsky, Saul  A.  and
%                     Vetterling, William  T. },
%       ISBN      = {0521431085},
%	MONTH     = {October},
%	PUBLISHER = {{Cambridge University Press}},
%	TITLE     = {Numerical Recipes in C : The Art of Scientific Computing},
%	YEAR      = {1992}
%      }
%

% precision
EPS = 3.0e-14;

% 1/\pi^{1/4}
PIM4 = 0.7511255444649425;

% maximum number of loops
MAXIT = 10;

% allocate the return values
X = zeros([N 1]);
W = zeros([N 1]);

    for i=1:(N+1)/2

      % good guesses at initial values for specific roots
      if i == 1
        z = sqrt(2.0*N+1.0) - 1.85575*((2.0*N+1)^(-0.16667));
      elseif i == 2
        z = z - (1.14 * N^0.426 / z);
      elseif i == 3
        z = 1.86 * z - 0.86 * X(1);
      elseif i == 4
        z = 1.91 * z - 0.91 * X(2);
      else
        z = 2.0*z - X(i-2);
      end

      for iter=1:MAXIT+1
        p1 = PIM4;
        p2 = 0.0;

        for j=1:N
          p3 = p2;
          p2 = p1;
          p1 = z * sqrt(2.0/j) * p2 - sqrt((j-1.0)/j) * p3;
        end

        % the derivative
        pp = sqrt(2.0*N) * p2;

        % newton step
        z1 = z;
        z  = z1 - p1/pp;

        if abs(z-z1) <= EPS
          break;
        end    
      end

      if iter == MAXIT+1
        fprintf('Too many iterations in hermquad.\n');
      end

      X(i)     = z;
      X(N+1-i) = -z;
      W(i)     = 2.0/(pp*pp);
      W(N+1-i) = W(i);

    end
end

 
