function summarize(all_estimates)

% This files reads in the outfile and 
% 1. Creates the tables for excel
% 2. Plots the estimated distributions of valuations
% 3. Estimates the distribution of the number of bidders and plots them

Kn = 4;
Kx = 3;
 
% Store results used for tables 

% Estimates of the semiparametric model (excluding distr. of order stat)
sem_opt = all_estimates(8:12,:);
% Summary statistics of the nonparametric model
np_opt = all_estimates(32:36,:);
 
% Standard errors of summary statistics
sem_se = all_estimates(45:49,:);
np_se = all_estimates(54:58,:);

sam_size = all_estimates(37,:);

% All estimated parameters of the nonparametric model
np_est = all_estimates(17:31,:); 

% Test for nonprametric median instead of mean.
np_opt_med = all_estimates(end-3:end-2,:);
np_se_med = all_estimates(end-1:end,:);

% Conditional order stat estimates used to estimate distribution of the
% number of bidders
np_cond  = np_est(12:15,:);
para_opt = all_estimates(1:7,:);
 
 
display_sp_est = zeros(2,6,3);
display_sp_se = zeros(2,6,3);

for carrier=1:3
        display_sp_est( 1 : 2  ,:,carrier) = [sem_opt(:,(carrier-1)*2+1:carrier*2)' sam_size(1,(carrier-1)*2+1:carrier*2)'];
        display_sp_se( 1 : 2 ,:,carrier) = [sem_se(:,(carrier-1)*2+1:carrier*2)' sam_size(1,(carrier-1)*2+1:carrier*2)'];
end


% Table 4 in the paper
disp('Table 4')

disp('Semiparametric estimates')

sp_est_c1 = [display_sp_est(:,1:5,1)' display_sp_est(2,1:5,1)'-display_sp_est(1,1:5,1)'];
sp_est_c2 = [display_sp_est(:,1:5,2)' display_sp_est(2,1:5,2)'-display_sp_est(1,1:5,2)'];
sp_est_c3 = [display_sp_est(:,1:5,3)' display_sp_est(2,1:5,3)'-display_sp_est(1,1:5,3)'];

sp_est_diff_in_diff = [sp_est_c3(:,3)-sp_est_c1(:,3) sp_est_c3(:,3)-sp_est_c2(:,3)];

sp_estimates_table_4 = [sp_est_c1 sp_est_c2 sp_est_c3 sp_est_diff_in_diff];
sp_estimates_table_4([2 3],:) = sp_estimates_table_4([3 2],:);

disp(sp_estimates_table_4)

disp('Semiparametric standard errors')

sp_se_c1 = [display_sp_se(:,1:5,1)' sqrt(display_sp_se(2,1:5,1).^2'+display_sp_se(1,1:5,1).^2')];
sp_se_c2 = [display_sp_se(:,1:5,2)' sqrt(display_sp_se(2,1:5,2).^2'+display_sp_se(1,1:5,2).^2')];
sp_se_c3 = [display_sp_se(:,1:5,3)' sqrt(display_sp_se(2,1:5,3).^2'+display_sp_se(1,1:5,3).^2')];

sp_se_diff_in_diff = [sqrt(sp_se_c3(:,3).^2+sp_se_c1(:,3).^2) sqrt(sp_se_c3(:,3).^2+sp_se_c2(:,3).^2)];

sp_standard_errors_table_4 = [sp_se_c1 sp_se_c2 sp_se_c3 sp_se_diff_in_diff];
sp_standard_errors_table_4([2 3],:) = sp_standard_errors_table_4([3 2],:);

disp(sp_standard_errors_table_4)


display_np_est = zeros(2,6,3);
display_np_se = zeros(2,6,3);

display_np_est_med = zeros(2,3,3);
display_np_se_med = zeros(2,3,3);


for carrier=1:3 
        display_np_est( : ,:,carrier) = [np_opt(:,(carrier-1)*2+1:carrier*2)' sam_size(1,(carrier-1)*2+1:carrier*2)'];
        display_np_se(: ,:,carrier) = [np_se(:,(carrier-1)*2+1:carrier*2)' sam_size(1,(carrier-1)*2+1:carrier*2)'];

        display_np_est_med(: ,:,carrier) = [np_opt_med(:,(carrier-1)*2+1:carrier*2)' sam_size(1,(carrier-1)*2+1:carrier*2)'];
        display_np_se_med(: ,:,carrier) = [np_se_med(:,(carrier-1)*2+1:carrier*2)' sam_size(1,(carrier-1)*2+1:carrier*2)'];
end


disp('Nonparametric estimates')

np_est_c1 = [display_np_est(:,1:5,1)' display_np_est(2,1:5,1)'-display_np_est(1,1:5,1)'];
np_est_c2 = [display_np_est(:,1:5,2)' display_np_est(2,1:5,2)'-display_np_est(1,1:5,2)'];
np_est_c3 = [display_np_est(:,1:5,3)' display_np_est(2,1:5,3)'-display_np_est(1,1:5,3)'];

np_est_diff_in_diff = [np_est_c3(:,3)-np_est_c1(:,3) np_est_c3(:,3)-np_est_c2(:,3)];

np_estimates_table_4 = [np_est_c1 np_est_c2 np_est_c3 np_est_diff_in_diff];
np_estimates_table_4([2 3],:) = np_estimates_table_4([3 2],:);

disp(np_estimates_table_4)
 
disp('Nonparametric standard errors')

np_se_c1 = [display_np_se(:,1:5,1)' sqrt(display_np_se(2,1:5,1).^2'+display_np_se(1,1:5,1).^2')];
np_se_c2 = [display_np_se(:,1:5,2)' sqrt(display_np_se(2,1:5,2).^2'+display_np_se(1,1:5,2).^2')];
np_se_c3 = [display_np_se(:,1:5,3)' sqrt(display_np_se(2,1:5,3).^2'+display_np_se(1,1:5,3).^2')];

 
np_se_diff_in_diff = [sqrt(np_se_c3(:,3).^2+np_se_c1(:,3).^2) sqrt(np_se_c3(:,3).^2+np_se_c2(:,3).^2)];

np_standard_errors_table_4 = [np_se_c1 np_se_c2 np_se_c3 np_se_diff_in_diff];
np_standard_errors_table_4([2 3],:) = np_standard_errors_table_4([3 2],:);

disp(np_standard_errors_table_4)
 


% Plot estimated densities for each time conditioning
   
M1 = intoneconst(Kn+1);
M1 = M1(2:end,1:Kn);

plotdistributionsall(np_est,sem_opt, Kn, Kx, M1)
 
% Now estimate and plot the number of bidders
nbidders( np_cond, para_opt,np_est,sem_opt, Kn, Kx, M1)
    

end


function M = intoneconst(J)

M = zeros(J,J);
for j=1:J
    for k=1:J
        if mod(j+k,2)==0
            M(j,k) = ((1/sqrt(2))^(j+k-2) )*sqrt(pi)*(dfact(j+k-2)) ;
        end
    end
end
    
end


function f = dfact(k)

k2 = k/2;
f=1;
for i=1:k2
	f = f*(2*i-1); 
end

end

function f = npdist(x,ax,mu,sig)

fx = zeros(size(x));

y = (x-mu)/sig;

for j=1:size(ax,1)
    for k=1:size(ax,1)
        fx = fx + ax(j,1)*ax(k,1)*intxex((j-1)+(k-1),y);
    end
end

f = fx.*(fx<=1) + ones(size(fx)).*(fx>1);

%f = fx;

end

% Calculates int^y_{-inf} x^p e^{-x^2}
function f = intxex(p,y)

if p == 0
	f = sqrt(pi)*normcdf(y,0,sqrt(1/2));
elseif p == 1
	f = (-1/2)*exp(-( y ).^2 );
else
    f = (p-1)/2*intxex(p-2,y) - (1/2)*y.^(p-1).*exp(-( y ).^2 );
end

end

function f = npdens(x,ax,mu,sig)

fx = zeros(size(x));

y = (x-mu)/sig;

for j=1:size(ax,1)
    fx = fx + (y).^(j-1)*ax(j,1);
end

f = (1/sig)*(fx.^2).*exp(-( y ).^2 );

end



function phat = sumstats(phatnp, Kn, Kx, M1, mu_v_est, sig_v_est, mu_w_est, sig_w_est, sig_x_est)

% Calculate summary stats
av = phatnp(1:Kn,1);
aw = phatnp(Kn+1:2*Kn,1);
ax = phatnp(2*Kn+1:2*Kn+Kx,1);

phat = zeros(5,1);

phat(1,1) = sig_v_est*av'*M1*av+mu_v_est;
phat(2,1) = sig_w_est*aw'*M1*aw+mu_w_est;

M2 = intoneconst(Kn+2);
M2 = M2(3:end,1:Kn);

phat(3,1) = sqrt(sig_v_est^2*av'*M2*av + mu_v_est^2 + 2*mu_v_est*sig_v_est*av'*M1*av - phat(1,1)^2);
phat(4,1) = sqrt(sig_w_est^2*aw'*M2*aw + mu_w_est^2 + 2*mu_w_est*sig_w_est*aw'*M1*aw - phat(2,1)^2);
phat(5,1) = sqrt(sig_x_est^2*ax'*M2(1:Kx,1:Kx)*ax - 0);

end
        
 


function plotdistributionsall(phatnpall,phat_3_all, Kn, Kx, M1)


num_spec = size(phatnpall,2); 
 
lv = zeros(num_spec,1);
uv = zeros(num_spec,1);

lw = zeros(num_spec,1);
uw = zeros(num_spec,1);

lx = zeros(num_spec,1);
ux = zeros(num_spec,1);

% num_spec is 6 for 3 carriers and 2 periods (pre and post)

for j = 1:num_spec

    phatnp = phatnpall(:,j);

    phat_3 = phat_3_all(:,j);
    
    mu_v_est = phat_3(1,1);
    sig_v_est = sqrt(2)*phat_3(3,1);  
    mu_w_est = phat_3(2,1);
    sig_w_est = sqrt(2)*phat_3(4,1);  
    sig_x_est = sqrt(2)*phat_3(5,1);  
    
    phat = sumstats(phatnp, Kn, Kx, M1, mu_v_est, sig_v_est, mu_w_est, sig_w_est, sig_x_est);

    % Find good lower and upper bound for figures based on means plus minus
    % 4 time the standard deviation
    lv(j,1) = floor(phat(1,1)-4*phat(3,1));
    uv(j,1) = ceil(phat(1,1)+4*phat(3,1));

    lw(j,1) = floor(phat(2,1)-4*phat(4,1));
    uw(j,1) = ceil(phat(2,1)+4*phat(4,1));
    
    lx(j,1) = floor(0-4*phat(5,1));
    ux(j,1) = ceil(0+4*phat(5,1));
    
end

% Make grids for plots
lv = min(lv);
uv = max(uv);

lw = min(lw)-1;
uw = max(uw);

lx = min(lx);
ux = max(ux);

xv = (lv:0.01:uv)';
xw = (lw:0.01:uw)';
xx = (lx:0.01:ux)';

npdv = zeros(size(xv,1),num_spec);
npdev = zeros(size(xv,1),num_spec);

npdw = zeros(size(xw,1),num_spec);
npdew = zeros(size(xw,1),num_spec);

npdx = zeros(size(xx,1),num_spec);
npdex = zeros(size(xx,1),num_spec);


% Calculate all 6 density and distribution functions

for j = 1:num_spec
    
    phatnp = phatnpall(:,j);
    
    phat_3 = phat_3_all(:,j);

    %mu_u2_est = phat_3(6,1);
    %sig_u2_est = sqrt(2)*phat_3(7,1);        
    mu_v_est = phat_3(1,1);
    sig_v_est = sqrt(2)*phat_3(3,1);  
    mu_w_est = phat_3(2,1);
    sig_w_est = sqrt(2)*phat_3(4,1);  
    sig_x_est = sqrt(2)*phat_3(5,1);  

    av = phatnp(1:Kn,1);
    aw = phatnp(Kn+1:2*Kn,1);
    ax = phatnp(2*Kn+1:2*Kn+Kx,1);
 
    npdv(:,j) = npdist(xv,av,mu_v_est,sig_v_est);
    npdev(:,j) = npdens(xv,av,mu_v_est,sig_v_est);
 
    npdw(:,j) = npdist(xw,aw,mu_w_est,sig_w_est);
    npdew(:,j) = npdens(xw,aw,mu_w_est,sig_w_est);

    npdx(:,j) = npdist(xx,ax,0,sig_x_est);
    npdex(:,j) = npdens(xx,ax,0,sig_x_est);
    

end 
 

% Figure 1

% All three carriers, pre period

fig = figure;

subplot(3,2,1);
plot(xv,npdv(:,1),xv, npdv(:,3), xv, npdv(:,5), 'linewidth', 2)
legend('AT&T Phones','Verizon Phones', 'Unlocked Phones', 'Location', 'NorthWest')
xlabel('$u$','Interpreter','latex','FontSize',12);
ylabel('$F_U(u)$','Interpreter','latex','FontSize',12);
xlim([-2 1])
ylim([0 1])

subplot(3,2,2);
plot(xv,npdev(:,1),xv, npdev(:,3), xv, npdev(:,5), 'linewidth', 2)
legend('AT&T Phones','Verizon Phones', 'Unlocked Phones', 'Location', 'NorthWest')
xlabel('$u$','Interpreter','latex','FontSize',12);
ylabel('$f_U(u)$','Interpreter','latex','FontSize',12);
set(gca,'YTick',(0:1:4)' );
xlim([-2 1])
ylim([0 4]) 

subplot(3,2,3);
plot(xw,npdw(:,1),xw, npdw(:,3), xw, npdw(:,5), 'linewidth', 2)
legend('AT&T Phones','Verizon Phones', 'Unlocked Phones', 'Location', 'NorthWest')
xlabel('$w$','Interpreter','latex','FontSize',12);
ylabel('$F_W(w)$','Interpreter','latex','FontSize',12);
set(gca,'XTick',(min(xw):1:max(xw))' );
xlim([-2 1])
ylim([0 1])

subplot(3,2,4);
plot(xw,npdew(:,1),xw, npdew(:,3), xw, npdew(:,5), 'linewidth', 2)
legend('AT&T Phones','Verizon Phones', 'Unlocked Phones', 'Location', 'NorthWest')
xlabel('$w$','Interpreter','latex','FontSize',12);
ylabel('$f_W(w)$','Interpreter','latex','FontSize',12);
set(gca,'XTick',(min(xw):1:max(xw))' );
set(gca,'YTick',(0:1:4)' );
xlim([-2 1])
ylim([0 4]) 

subplot(3,2,5);
plot(xx,npdx(:,1),xx, npdx(:,3), xx, npdx(:,5), 'linewidth', 2)
legend('AT&T Phones','Verizon Phones', 'Unlocked Phones', 'Location', 'NorthWest')
xlabel('$x$','Interpreter','latex','FontSize',12);
ylabel('$F_X(x)$','Interpreter','latex','FontSize',12);
set(gca,'XTick',(min(xx):1:max(xx))' );
xlim([-1 1])
ylim([0 1])

subplot(3,2,6);
plot(xx,npdex(:,1),xx, npdex(:,3), xx, npdex(:,5), 'linewidth', 2)
legend('AT&T Phones','Verizon Phones', 'Unlocked Phones', 'Location', 'NorthWest')
xlabel('$x$','Interpreter','latex','FontSize',12);
ylabel('$f_X(x)$','Interpreter','latex','FontSize',12);
set(gca,'XTick',(min(xx):1:max(xx))' );
set(gca,'YTick',(0:2:8)' );
xlim([-1 1])
ylim([0 8])


set(fig, 'Position', [0, 0, 1100, 900]);

str=sprintf('Figure_1.pdf');

h=gcf;
set(h,'PaperOrientation','landscape');
saveas(h,str)


fig = figure;

subplot(3,2,1);
plot(xv,npdv(:,2),xv, npdv(:,4), xv, npdv(:,6), 'linewidth', 2)
legend('AT&T Phones','Verizon Phones', 'Unlocked Phones', 'Location', 'NorthWest')
xlabel('$u$','Interpreter','latex','FontSize',12);
ylabel('$F_U(u)$','Interpreter','latex','FontSize',12);
xlim([-2 1])
ylim([0 1])

subplot(3,2,2);
plot(xv,npdev(:,2),xv, npdev(:,4), xv, npdev(:,6), 'linewidth', 2)
legend('AT&T Phones','Verizon Phones', 'Unlocked Phones', 'Location', 'NorthWest')
xlabel('$u$','Interpreter','latex','FontSize',12);
ylabel('$f_U(u)$','Interpreter','latex','FontSize',12);
set(gca,'YTick',(0:1:4)' );
xlim([-2 1])
ylim([0 4]) 

subplot(3,2,3);
plot(xw,npdw(:,2),xw, npdw(:,4), xw, npdw(:,6), 'linewidth', 2)
legend('AT&T Phones','Verizon Phones', 'Unlocked Phones', 'Location', 'NorthWest')
xlabel('$w$','Interpreter','latex','FontSize',12);
ylabel('$F_W(w)$','Interpreter','latex','FontSize',12);
set(gca,'XTick',(min(xw):1:max(xw))' );
xlim([-2 1])
ylim([0 1])

subplot(3,2,4);
plot(xw,npdew(:,2),xw, npdew(:,4), xw, npdew(:,6), 'linewidth', 2)
legend('AT&T Phones','Verizon Phones', 'Unlocked Phones', 'Location', 'NorthWest')
xlabel('$w$','Interpreter','latex','FontSize',12);
ylabel('$f_W(w)$','Interpreter','latex','FontSize',12);
set(gca,'XTick',(min(xw):1:max(xw))' );
set(gca,'YTick',(0:1:4)' );
xlim([-2 1])
ylim([0 4]) 

subplot(3,2,5);
plot(xx,npdx(:,2),xx, npdx(:,4), xx, npdx(:,6), 'linewidth', 2)
legend('AT&T Phones','Verizon Phones', 'Unlocked Phones', 'Location', 'NorthWest')
xlabel('$x$','Interpreter','latex','FontSize',12);
ylabel('$F_X(x)$','Interpreter','latex','FontSize',12);
set(gca,'XTick',(min(xx):1:max(xx))' );
xlim([-1 1])
ylim([0 1])

subplot(3,2,6);
plot(xx,npdex(:,2),xx, npdex(:,4), xx, npdex(:,6), 'linewidth', 2)
legend('AT&T Phones','Verizon Phones', 'Unlocked Phones', 'Location', 'NorthWest')
xlabel('$x$','Interpreter','latex','FontSize',12);
ylabel('$f_X(x)$','Interpreter','latex','FontSize',12);
set(gca,'XTick',(min(xx):1:max(xx))' );
set(gca,'YTick',(0:2:8)' );
xlim([-1 1])
ylim([0 8])


set(fig, 'Position', [0, 0, 1100, 900]);
str=sprintf('Figure_2.pdf');

h=gcf;
set(h,'PaperOrientation','landscape');
saveas(h,str)


 

% Figure 3

fig = figure;

subplot(3,3,1);
plot(xv,npdev(:,1),xv, npdev(:,2), 'linewidth', 2)
legend('Pre','Post', 'Location', 'NorthWest')
xlabel('$u$','Interpreter','latex','FontSize',12);
ylabel('$f_U(u)$','Interpreter','latex','FontSize',12);
set(gca,'YTick',(0:1:4)' );
xlim([-2 1])
ylim([0 4])  
title({'AT\&T Phones';''},'Interpreter','latex','FontSize',14)



subplot(3,3,2);
plot(xv,npdev(:,3),xv, npdev(:,4) , 'linewidth', 2)
legend('Pre','Post', 'Location', 'NorthWest')
xlabel('$u$','Interpreter','latex','FontSize',12);
ylabel('$f_U(u)$','Interpreter','latex','FontSize',12);
set(gca,'YTick',(0:1:4)' );
xlim([-2 1])
ylim([0 4])  
title({'Verizon Phones';''},'Interpreter','latex','FontSize',14)


subplot(3,3,3);
plot(xv,npdev(:,5),xv, npdev(:,6) , 'linewidth', 2)
legend('Pre','Post', 'Location', 'NorthWest')
xlabel('$u$','Interpreter','latex','FontSize',12);
ylabel('$f_U(u)$','Interpreter','latex','FontSize',12);
set(gca,'YTick',(0:1:4)' );
xlim([-2 1])
ylim([0 4])  
title({'Unlocked Phones';''},'Interpreter','latex','FontSize',14)


subplot(3,3,4);
plot(xw,npdew(:,1),xw,npdew(:,2), 'linewidth', 2)
legend('Pre','Post', 'Location', 'NorthWest')
xlabel('$w$','Interpreter','latex','FontSize',12);
ylabel('$f_W(w)$','Interpreter','latex','FontSize',12);
set(gca,'XTick',(min(xw):1:max(xw))' );
set(gca,'YTick',(0:1:4)' );
xlim([-2 1])
ylim([0 4]) 
 


subplot(3,3,5);
plot(xw,npdew(:,3),xw,npdew(:,4) , 'linewidth', 2)
legend('Pre','Post', 'Location', 'NorthWest')
xlabel('$w$','Interpreter','latex','FontSize',12);
ylabel('$f_W(w)$','Interpreter','latex','FontSize',12);
set(gca,'XTick',(min(xw):1:max(xw))' );
set(gca,'YTick',(0:1:4)' );
xlim([-2 1])
ylim([0 4]) 
 

subplot(3,3,6);
plot(xw,npdew(:,5),xw,npdew(:,6) , 'linewidth', 2)
legend('Pre','Post', 'Location', 'NorthWest')
xlabel('$w$','Interpreter','latex','FontSize',12);
ylabel('$f_W(w)$','Interpreter','latex','FontSize',12);
set(gca,'XTick',(min(xw):1:max(xw))' );
set(gca,'YTick',(0:1:4)' );
xlim([-2 1])
ylim([0 4])  



subplot(3,3,7);
plot(xx,npdex(:,1),xx,npdex(:,2), 'linewidth', 2)
legend('Pre','Post', 'Location', 'NorthWest')
xlabel('$x$','Interpreter','latex','FontSize',12);
ylabel('$f_X(x)$','Interpreter','latex','FontSize',12);
set(gca,'XTick',(min(xx):1:max(xx))' );
set(gca,'YTick',(0:2:8)' );
xlim([-1 1])
ylim([0 8])

 


subplot(3,3,8);
plot(xx,npdex(:,3),xx,npdex(:,4) , 'linewidth', 2)
legend('Pre','Post', 'Location', 'NorthWest')
xlabel('$x$','Interpreter','latex','FontSize',12);
ylabel('$f_X(x)$','Interpreter','latex','FontSize',12);
set(gca,'XTick',(min(xx):1:max(xx))' );
set(gca,'YTick',(0:2:8)' );
xlim([-1 1])
ylim([0 8])

 

subplot(3,3,9);
plot(xx,npdex(:,5),xx,npdex(:,6) , 'linewidth', 2)
legend('Pre','Post', 'Location', 'NorthWest')
xlabel('$x$','Interpreter','latex','FontSize',12);
ylabel('$f_X(x)$','Interpreter','latex','FontSize',12);
set(gca,'XTick',(min(xx):1:max(xx))' );
set(gca,'YTick',(0:2:8)' );
xlim([-1 1])
ylim([0 8])

 
set(fig, 'Position', [0, 0, 1200, 850]);
str=sprintf('Figure_3.pdf');

h=gcf;
set(h,'PaperOrientation','landscape');
saveas(h,str)


end





function nbidders(np_cond, para_opt, phatnpall,phat_3_all, Kn, Kx, M1)

options = optimset('Display','final','MaxFunEval', 100000,'MaxIter', 100000 );

num_spec = size(phatnpall,2); 


% Ee need to estimate the distribution functions of valuations and the
% second order stat. To to so, we first a grid based on the mean plus/minus
% 4 times the standard deviation

lv = zeros(num_spec,1);
uv = zeros(num_spec,1);
    
for j = 1:num_spec

    phatnp = phatnpall(:,j);

    phat_3 = phat_3_all(:,j);

    mu_v_est = phat_3(1,1);
    sig_v_est = sqrt(2)*phat_3(3,1);  
    mu_w_est = phat_3(2,1);
    sig_w_est = sqrt(2)*phat_3(4,1);  
    sig_x_est = sqrt(2)*phat_3(5,1);  
    
    phat = sumstats(phatnp, Kn, Kx, M1, mu_v_est, sig_v_est, mu_w_est, sig_w_est, sig_x_est);
    
    lv(j,1) = floor(phat(1,1)-4*phat(3,1));
    uv(j,1) = ceil(phat(1,1)+4*phat(3,1));
 
end

lv = min(lv);
uv = max(uv);
 
xv = (lv:0.01:uv)';
 
npdu2 = zeros(size(xv,1),num_spec);
npdv = zeros(size(xv,1),num_spec);

% Possibly semiparametric estimates

% For each carrier and periods (pre and post, we estimate the distribution
% of the number of bidders parametrically and nonparametrically.

% Maximum number of bidders for each case 
% (carrier 1, pre), (carrier 1, post), (carrier 2, pre).... 
max_bidders_vec = [1000;1000;1000;1000;1000;10000];

pmf_para = zeros(max(max_bidders_vec)-2, num_spec);
pmf_npara = zeros(max(max_bidders_vec)-2, num_spec);

cdf_para = ones(max(max_bidders_vec)-2, num_spec);
cdf_npara = ones(max(max_bidders_vec)-2, num_spec);

for j = 1:num_spec
    
    phatnp = phatnpall(:,j);
    phat_3 = phat_3_all(:,j);
    phat_p = para_opt(:,j);
   
    
    % Estimate cdf of valuation and second order statistic. 
    
    mu_v_est = phat_3(1,1);
    sig_v_est = sqrt(2)*phat_3(3,1);  
    av = phatnp(1:Kn,1);

    mu_u2_est = phat_p(6,1);
    sig_u2_est = sqrt(2)*phat_p(7,1);  
    au2_np =  np_cond(:,j);
    
    npdu2(:,j) = npdist(xv,au2_np,mu_u2_est,sig_u2_est);
    npdv(:,j) = npdist(xv,av,mu_v_est,sig_v_est);
    
    % Grid on which we estimate. Exclude values close to 0 and 1.
    index = ( ( (npdv(:,j)>=1e-5).*(npdv(:,j)<=1-1e-5) ) + ( (npdu2(:,j)>=1e-5).*(npdu2(:,j)<=1-1e-5) ))>0;
    
    % Different upper bound for the number of bidders
    
        max_bidders = max_bidders_vec(j,1);
     
    
        bidders_vec = (3:max_bidders)';
    
        reg_mat = zeros(size(npdv(index,j),1),size(bidders_vec,1));

        for n = bidders_vec(1,1):bidders_vec(end,1)
            for i = n-2:n
                reg_mat(:,n-2) = reg_mat(:,n-2)+ nchoosek(n,i)*npdv(index,j).^i.*(1-npdv(index,j)).^(n-i);
            end
        end


        % Starting values parametric model
        lambda_vec = [10;20;50;100;150;200;300;400];
        p_vec = ones(size(lambda_vec,1),1)/size(lambda_vec,1);
        params_start = [lambda_vec;p_vec];

        % Constrains paramtric model
        Aeq = [zeros(1,size(lambda_vec,1)) ones(1,size(p_vec,1))];
        beq = 1;

        [xp,~] = fmincon(@(params)objnb( params , npdu2(index,j), reg_mat, bidders_vec),params_start,[],[],Aeq,beq,zeros(size(params_start)),[ones(size(lambda_vec))*10000;ones(size(p_vec) )],[],options );
        pmf_para(1:max_bidders-2,j) = est_dens(xp, bidders_vec);
    
        [x,~]  = quadprog((reg_mat'*reg_mat), -(npdu2(index,j)'*reg_mat)',[],[],ones(size(bidders_vec,1),1)',1,zeros(size(bidders_vec,1),1), ones(size(bidders_vec,1),1) );
        pmf_npara(1:max_bidders-2,j) = x;
 
        cdf_para(1:max_bidders-2,j) = cumsum(pmf_para(1:max_bidders-2,j)); 

        cdf_npara(1:max_bidders-2,j) = cumsum(pmf_npara(1:max_bidders-2,j)); 
    
        
end


% Figure 4 

fig = figure;


% Plots Carrier 1

plot_index_1 = ceil(sum( ( (cdf_para(:,1) <= 0.99)+(cdf_npara(:,1) <= 0.99) + (cdf_para(:,2) <= 0.99)+(cdf_npara(:,2) <= 0.99) >0))/100)*100;

subplot(3,2,1);
plot(3:plot_index_1,cdf_npara(1:plot_index_1-2,1),3:plot_index_1,cdf_para(1:plot_index_1-2,1) , 'linewidth', 2)
legend('Nonprametric CDF', 'Parametric CDF',  'Location', 'SouthEast')
ylim([0 1]) 
xlabel('$n$','Interpreter','latex','FontSize',12);

h1 = text(-plot_index_1/5, 0.4,'AT\&T','Interpreter','latex','FontSize',12);
set(h1, 'rotation', 90)
h2 = text(plot_index_1*0.9/2, 1.2,'Pre','Interpreter','latex','FontSize',12);

subplot(3,2,2);
plot(3:plot_index_1,cdf_npara(1:plot_index_1-2,2),3:plot_index_1,cdf_para(1:plot_index_1-2,2) , 'linewidth', 2)
legend('Nonprametric CDF', 'Parametric CDF',  'Location', 'SouthEast')
ylim([0 1]) 
xlabel('$n$','Interpreter','latex','FontSize',12);

h3 = text(plot_index_1*0.9/2, 1.2,'Post','Interpreter','latex','FontSize',12);


% Plots Carrier 2

plot_index_2 = ceil(sum( ( (cdf_para(:,3) <= 0.99)+(cdf_npara(:,3) <= 0.99) + (cdf_para(:,4) <= 0.99)+(cdf_npara(:,4) <= 0.99) >0))/100)*100;

subplot(3,2,3);
plot(3:plot_index_2,cdf_npara(1:plot_index_2-2,3),3:plot_index_2,cdf_para(1:plot_index_2-2,3) , 'linewidth', 2)
legend('Nonprametric CDF', 'Parametric CDF',  'Location', 'SouthEast')
ylim([0 1]) 
xlabel('$n$','Interpreter','latex','FontSize',12);

h4 = text(-plot_index_2/5, 0.3,'Verizon','Interpreter','latex','FontSize',12);
set(h4, 'rotation', 90)

subplot(3,2,4);
plot(3:plot_index_2,cdf_npara(1:plot_index_2-2,4),3:plot_index_2,cdf_para(1:plot_index_2-2,4) , 'linewidth', 2)
legend('Nonprametric CDF', 'Parametric CDF',  'Location', 'SouthEast')
ylim([0 1]) 
xlabel('$n$','Interpreter','latex','FontSize',12);

 
% Plots Carrier 3

plot_index_3_pre =  ceil(sum( ( (cdf_para(:,5) <= 0.99)+(cdf_npara(:,5) <= 0.99) > 0 ))/1000)*1000;
plot_index_3_post = ceil(sum( ( (cdf_para(:,6) <= 0.99)+(cdf_npara(:,6) <= 0.99) > 0 ))/1000)*1000;

subplot(3,2,5);
plot(3:plot_index_3_pre,cdf_npara(1:plot_index_3_pre-2,5),3:plot_index_3_pre,cdf_para(1:plot_index_3_pre-2,5) , 'linewidth', 2)
legend('Nonprametric CDF', 'Parametric CDF',  'Location', 'SouthEast')
ylim([0 1]) 
xlabel('$n$','Interpreter','latex','FontSize',12);

h5 = text(-plot_index_3_pre/5, 0.3,'Unlocked','Interpreter','latex','FontSize',12);
set(h5, 'rotation', 90)

subplot(3,2,6);
plot(3:plot_index_3_post,cdf_npara(1:plot_index_3_post-2,6),3:plot_index_3_post,cdf_para(1:plot_index_3_post-2,6) , 'linewidth', 2)
legend('Nonprametric CDF', 'Parametric CDF',  'Location', 'SouthEast')
ylim([0 1]) 
xlabel('$n$','Interpreter','latex','FontSize',12);
        
 
set(fig, 'Position', [0, 0, 1000, 800]);

str=sprintf('Figure_4.pdf');

h=gcf;

set(h,'PaperOrientation','landscape');

saveas(h,str)
    

 

end

        
function f  = objnb(params, npdu2temp, reg_mat, bidders_vec)
 
    lambda_vec = params(1:size(params,1)/2,1);
    p_vec= params( size(params,1)/2 + 1:end,1);
    
    temp = zeros(size(bidders_vec,1),size(lambda_vec,1));
    for jp = 1:size(lambda_vec,1)
        
        temp(:,jp) =  poisspdf(bidders_vec-3,lambda_vec(jp,1));
        
     end 

    f = sum( (npdu2temp - reg_mat*(temp*p_vec)).^2);
    

end

 
% Linear combination of poissons
function f  = est_dens(params, bidders_vec)
 
    lambda_vec = params(1:size(params,1)/2,1);
    p_vec= params( size(params,1)/2 + 1:end,1);
    
    temp = zeros(size(bidders_vec,1),size(lambda_vec,1));
    for jp = 1:size(lambda_vec,1)
        
        temp(:,jp) =  poisspdf(bidders_vec-3,lambda_vec(jp,1));
 
    end 
     
    f = temp*p_vec;

end

        
 
